﻿using CtrEditor.Convertidores;
using CtrEditor.Simulacion;
using CtrEditor.Siemens;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;


namespace CtrEditor.ObjetosSim.UserControls
{
    /// <summary>
    /// Interaction logic for ucPhotocell.xaml
    /// </summary>
    public class osPhotocell : osBase, IosBase
    {
        private float _ancho;
        private float _altoPhotocell;
        private float _left;
        private float _top;
        private float _angulo;
        private string _nombre = NombreClase();
        private bool _luzCortada;
        private string _tagPhotocellOUT;
        private bool _tipoNC;
        private Brush _color;

        private simBarrera Simulation_Photocell;

        public static string NombreClase()
        {
            return "Photocell";
        }

        public Brush Color
        {
            get => _color;
            set
            {
                _color = value;
                OnPropertyChanged(nameof(Color));
            }
        }
        public bool LuzCortada
        {
            get => _luzCortada;
            set
            {
                if (_luzCortada != value)
                {
                    _luzCortada = value;
                    if (_luzCortada)
                        Color = Brushes.Blue;
                    else
                        Color = Brushes.Green;
                    OnPropertyChanged(nameof(LuzCortada));
                }
            }
        }

        public bool Tipo_NC
        {
            get => _tipoNC;
            set
            {
                if (_tipoNC != value)
                {
                    _tipoNC = value;
                    OnPropertyChanged(nameof(Tipo_NC));
                }
            }
        }

        public string TagPhotocell_OUT
        {
            get => _tagPhotocellOUT;
            set
            {
                if (_tagPhotocellOUT != value)
                {
                    _tagPhotocellOUT = value;
                    OnPropertyChanged(nameof(TagPhotocell_OUT));
                }
            }
        }

        public override float Left
        {
            get => _left;
            set
            {
                _left = value;
                ActualizarGeometrias();
                CanvasSetLeftinMeter(value);
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float Top
        {
            get => _top;
            set
            {
                _top = value;
                ActualizarGeometrias();
                CanvasSetTopinMeter(value);
                OnPropertyChanged(nameof(Top));
            }
        }

        public float Ancho
        {
            get => _ancho;
            set
            {
                _ancho = value;
                ActualizarGeometrias();
                OnPropertyChanged(nameof(Ancho));
            }
        }

        public float Alto
        {
            get => _altoPhotocell;
            set
            {
                _altoPhotocell = value;
                ActualizarGeometrias();
                OnPropertyChanged(nameof(Alto));
            }
        }

        public float Angulo
        {
            get => _angulo;
            set
            {
                _angulo = value;
                ActualizarGeometrias();
                OnPropertyChanged(nameof(Angulo));
            }
        }

        public override string Nombre
        {
            get => _nombre;
            set
            {
                if (_nombre != value)
                {
                    _nombre = value;
                    OnPropertyChanged(nameof(Nombre));
                }
            }
        }

        private void ActualizarGeometrias()
        {
            if (_visualRepresentation is ucPhotocell uc)
                UpdateRectangle(Simulation_Photocell, uc.Photocell, Alto, Ancho, Angulo);
        }

        public osPhotocell()
        {
            Ancho = 1;
            Alto = 0.03f;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }
        public override void UpdateControl(int elapsedMilliseconds)
        {
            LuzCortada = Simulation_Photocell.LuzCortada;
        }
        public override void UpdateGeometryStep()
        {
            Simulation_Photocell.LuzCortada = false;
        }
        public override void UpdatePLC(PLCModel plc, int elapsedMilliseconds) {
            EscribirBitTag(plc, TagPhotocell_OUT, LuzCortada);
        }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            ActualizarLeftTop();

            if (_visualRepresentation is ucPhotocell uc)
                Simulation_Photocell = AddBarrera(simulationManager, uc.Photocell, Alto, Ancho, Angulo);
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            simulationManager.Remove(Simulation_Photocell);
        }


    }

    public partial class ucPhotocell : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucPhotocell()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Resize(float width, float height)
        {
            if (width == 0) return;
            if (Datos is osPhotocell datos)
                datos.Ancho = PixelToMeter.Instance.calc.PixelsToMeters(width);
        }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle)
        {
            if (Datos != null)
                if (Datos is osPhotocell datos)
                    datos.Angulo = Angle;
        }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 16;
        }

    }
}
